@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2021 Elyra Authors
+ * Copyright 2018-2022 Elyra Authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,12 +57,7 @@ describe('@elyra/services', () => {
         const schemaNames = schemas.map((schema: any) => {
           return schema.name;
         });
-        const knownSchemaNames = [
-          'code-snippet',
-          'runtime-image',
-          'kfp',
-          'airflow'
-        ];
+        const knownSchemaNames = ['code-snippet', 'runtime-image'];
         for (const schemaName of knownSchemaNames) {
           expect(schemaNames).toContain(schemaName);
         }
